# (c) 2009 Jiri Baum
import time
import cPickle as pickle
from functools import partial

import tubes
from utils import *

class FeedModel():
    def __init__(self):
        self.views = []
        self.tubes = []
        self.my_updates = []
        self.others_updates = {}
        filename = cfgfilename('statuses')
        try:
            u = pickle.Unpickler(file(filename, 'rb'))
        except IOError:
            pass
        else:
            try:
                while True:
                    self.my_updates.append(u.load())
            except EOFError:
                pass
        self.store = file(filename, 'ab')

    def status_update(self, text):
        now = time.time()
        for v in self.views:
            v.new_item(text, by='me', time=now, buddy=tubes.me_information)
        for me in self.tubes:
            me.StatusUpdate(text, now)
        self.my_updates.append((text, now))
        pickle.dump((text, now), self.store)
        self.store.flush()

    def subscribe_view(self, view):
        self.views.append(view)
        for (text, when) in self.my_updates:
            view.new_item(text, by='me', time=when,
                buddy=tubes.me_information, bulk=True)
        for (name, text, when), buddy in self.others_updates.items():
            view.new_item(text, by=name, time=when, buddy=buddy, bulk=True)
        view.new_item_bulk_finish()

    def unsubscribe_view(self, view):
        self.views.remove(view)

    def subscribe_tube(self, tube, me, other, buddy):
        self.tubes.append(me)
        tube.add_signal_receiver(partial(self.receive_update, buddy),
            dbus_interface=tubes.SERVICE, signal_name='StatusUpdate')
        for (text, when) in self.my_updates:
            me.StatusUpdate(text, when)

    def receive_update(self, buddy, text, when):
        name = buddy.name()
        if (name, text, when) not in self.others_updates:
            self.others_updates[(name, text, when)] = buddy
            for v in self.views:
                v.new_item(text, by=name, time=when, buddy=buddy)

if __name__ == "__main__":
    import feed_v, gtk
    model = FeedModel()
    view = feed_v.FeedView(model)
    gtk.main()
