#!/usr/bin/python
# (c) 2009 Jiri Baum

from __future__ import with_statement

import os, re, tempfile
import pygtk
pygtk.require('2.0')
import glib, gtk, pango

from time import time as now
from utils import *

retweet_chars = set(unichr(x) for x in range(0x2672, 0x267D+1))
retweet_prefix = u'\u267A '
assert retweet_prefix.strip()[0] in retweet_chars

link_pat = r'\b((?:https?|ftp)://[\w\d:#@%/;$()~_?\+-=\\\.&]+\b/*)'
link_pat = re.compile(link_pat)

class FeedView:
    win_pos = None
    win_size = (400, 500)

    def __init__(self, model):
        self.model = model
        self.updaters = []

        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.window.set_default_size(*FeedView.win_size)
        if FeedView.win_pos:
            self.window.move(*FeedView.win_pos)
        icon_file = '/usr/share/icons/hicolor/scalable/apps/connectr.svg'
        self.window.set_icon_from_file(icon_file)
        self.window.connect("destroy", self.destroy)
        self.window.connect("delete_event", lambda *x: False)
        self.window.connect("configure_event", self.configure_event)
        self.quitter = quitter()

        main_box = gtk.VBox()
        self.window.add(main_box)

        self.update_exp = gtk.Expander('Update your status:')
        main_box.pack_start(self.update_exp, expand=False)
        update_box = gtk.HBox()
        update_btn_box = gtk.VBox()
        self.update_exp.add(update_box)
        self.update = gtk.TextView()
        self.update.set_wrap_mode(gtk.WRAP_WORD)
        self.update.set_accepts_tab(False)
        self.update.get_buffer().connect('changed', self.update_changed)
        self.update_btn = gtk.Button('Update')
        self.update_btn.set_sensitive(0)
        self.update_btn.connect("clicked", self.status_update)
        update_box.pack_start(self.update, expand=True)
        update_btn_box.pack_end(self.update_btn, expand=False)
        update_btn_eb = gtk.EventBox()
        update_btn_eb.add(update_btn_box)
        update_btn_eb.modify_bg(gtk.STATE_NORMAL,
                          update_btn_eb.get_style().base[gtk.STATE_NORMAL])
        update_box.pack_start(update_btn_eb, expand=False)
        self.update_exp.connect('notify::expanded', self.focus_update)

        scroller = gtk.ScrolledWindow()
        main_box.pack_start(scroller)
        self.feed_box = gtk.VBox(spacing=1)
        self.feed_list = []
        scroller.add_with_viewport(self.feed_box)
        self.window.show_all()

        self.model.subscribe_view(self)

    def focus_update(self, exp, exp_comment):
        if exp.get_expanded():
            self.update.grab_focus()

    def status_update(self, *args):
        update_buf = self.update.get_buffer()
        text = update_buf.get_text(*update_buf.get_bounds())
        text = text.strip()
        self.model.status_update(text)
        update_buf.set_text('')
        retweet_start = update_buf.get_mark('retweet-start')
        if retweet_start:
            update_buf.delete_mark(retweet_start)
        self.update_exp.set_expanded(False)

    def update_changed(self, *args):
        update_buf = self.update.get_buffer()
        text = update_buf.get_text(*update_buf.get_bounds())
        text = text.strip()
        if text:
            self.update_btn.set_sensitive(True)
        else:
            self.update_btn.set_sensitive(False)

    def new_item(self, text, by=None, time=None, buddy=None, bulk=False):
        feed_item = gtk.HBox()
        feed_itemtext = gtk.TextView()
        feed_itemtext.set_wrap_mode(gtk.WRAP_WORD)
        feed_itemtext.set_editable(False)
        feed_itemtext.set_cursor_visible(False)
        feed_itemtext.set_property('can-focus', False)
        buf = feed_itemtext.get_buffer()
        text = bidi_protect(text)
        link_tag = None
        for i, piece in enumerate(link_pat.split(text)):
            if i%2==0:
                # plain text
                buf.insert(buf.get_end_iter(), piece)
            else:
                # URL
                link_tag = buf.create_tag(underline=pango.UNDERLINE_SINGLE)
                # Insert ZWSP to allow more frequent word-breaks than GTK
                # allows. Might have used wrap_mode=gtk.WRAP_CHAR on the
                # tag but it applies to whole paragraphs at a time.
                buf.insert_with_tags(buf.get_end_iter(),
                    re.sub(r'\b', u'\u200B', piece), link_tag)
                link_tag.connect("event", self.link_cb, piece)

        if link_tag is not None:
            feed_itemtext.connect("enter-notify-event", self.reset_cursor)
            feed_itemtext.connect("leave-notify-event", self.reset_cursor)
            feed_itemtext.connect("motion-notify-event", self.reset_cursor)

        buf.insert(buf.get_end_iter(), ' ')
        if by is not None:
            by = bidi_protect(by)
            if unicode(text).strip().strip(u'\u200E\u200F')[0] in retweet_chars:
                buf.insert(buf.get_start_iter(), u' ')
            else:
                buf.insert(buf.get_start_iter(), 
                  {'L': u' \u25B7 ',
                   'R': u' \u25C1 '}[bidi_para_dir(by+u' '+text)]
                )
                # characters:
                #    for left-to-right paragrah-level text:
                #       25B7 - WHITE RIGHT-POINTING TRIANGLE
                #    for right-to-left paragrah-level text:
                #       25C1 - WHITE LEFT-POINTING TRIANGLE
                # Note: since nick has been bidi_protected, it'll always be
                # the nick that provides the direction, even if it's
                # neutral. Not sure what's the Right Thing there.
            buf.insert_with_tags(buf.get_start_iter(), by, 
                buf.create_tag(weight=pango.WEIGHT_BOLD))

            # Avoid the time getting in the middle of the text inserted so
            # far...
            if bidi_para_dir(by)!=bidi_para_dir(text):
                buf.insert(buf.get_end_iter(), 
                    {'L': u'\u200E', 'R': u'\u200F'}[bidi_para_dir(by)])

        retweet_text = buf.get_text(*buf.get_bounds())
        self.updaters.append(TimeUpdater(feed_itemtext, time))
        feed_item.pack_start(feed_itemtext)
        if buddy is not None:
            buddy.avatar_rq(partial(self.show_avatar, feed_item))

        menu_bar, menu = self.build_item_menu(retweet_text)
        vbox = gtk.VBox()
        vbox.pack_start(menu_bar, expand=False)
        eb = gtk.EventBox()
        eb.add(vbox)
        eb.modify_bg(gtk.STATE_NORMAL,
                                     eb.get_style().base[gtk.STATE_NORMAL])
        feed_item.pack_start(eb, expand=False)

        feed_item.connect("button-press-event", self.item_popup_menu, menu)
        feed_itemtext.connect("button-press-event", self.item_popup_menu, menu)

        # add item to item_list
        self.feed_box.pack_start(feed_item, expand=False)
        self.feed_list.append((time, feed_item))
        if not bulk:
            self.feed_list.sort(reverse=True)
            for i, (time, item) in enumerate(self.feed_list):
                self.feed_box.reorder_child(item, i)
            feed_item.show_all()

    def new_item_bulk_finish(self):
        self.feed_list.sort(reverse=True)
        for i, (time, item) in enumerate(self.feed_list):
            self.feed_box.reorder_child(item, i)
        self.feed_box.show_all()

    def link_cb(self, texttag, widget, event, iter, url):
        if event.type==gtk.gdk.BUTTON_PRESS and event.button==1:
            return gtk.show_uri(None, url, event.time)
        if event.type==gtk.gdk.MOTION_NOTIFY:
            widget.get_window(gtk.TEXT_WINDOW_TEXT).set_cursor(
                                             gtk.gdk.Cursor(gtk.gdk.HAND2))
            return True
    def reset_cursor(self, widget, event):
        widget.get_window(gtk.TEXT_WINDOW_TEXT).set_cursor(
                                             gtk.gdk.Cursor(gtk.gdk.XTERM))


    def build_item_menu(self, retweet_text):
        menu = gtk.Menu()

        for item in (
            (retweet_prefix.strip(), self.retweet, retweet_text),
            #None, # separator
        ):
            if item==None:
                menu.add(gtk.SeparatorMenuItem())
            else:
                text, callback, param = item
                mi = gtk.MenuItem(text)
                mi.connect("activate", callback, param)
                menu.add(mi)

        menu_bar = gtk.MenuBar()
        mi = gtk.MenuItem()
        mi.add(gtk.Arrow(gtk.ARROW_DOWN, gtk.SHADOW_OUT))
        mi.set_submenu(menu)
        menu_bar.append(mi)

        return menu_bar, menu

    def item_popup_menu(self, widget, event, menu):
        if event.button==3:
            menu.popup(None, None, None, event.button, event.time, None)
            return True

    def show_avatar(self, feed_item, type, imgdata):
        with tempfile.NamedTemporaryFile() as tf:
            imgdata = ''.join([chr(b) for b in imgdata])
            tf.write(imgdata)
            tf.flush()
            avatar = gtk.image_new_from_file(tf.name)

        vbox = gtk.VBox()
        vbox.pack_start(avatar, expand=False)

        eb = gtk.EventBox()
        eb.add(vbox)

        eb.modify_bg(gtk.STATE_NORMAL,
                                     eb.get_style().base[gtk.STATE_NORMAL])

        feed_item.pack_start(eb, expand=False)
        feed_item.reorder_child(eb, 0)
        eb.show_all()

    def retweet(self, menuitem, retweet_text):
        self.update_exp.set_expanded(True)
        buf = self.update.get_buffer()

        # Is there already a retweet in progress? If so, replace it.
        prev_retweet = buf.get_mark('retweet-start')
        if prev_retweet:
            buf.delete(buf.get_iter_at_mark(prev_retweet), buf.get_end_iter())
            buf.delete_mark(prev_retweet)

        # Mark the place where we'll put the cursor.
        mark_c = buf.create_mark(None, buf.get_end_iter(), left_gravity=True)
        text = buf.get_text(*buf.get_bounds())
        if text and not text[-1].isspace():
            buf.insert(buf.get_end_iter(), ' ')
        # Mark the place from which retweet replacement will work.
        mark_rts = buf.create_mark(None, buf.get_end_iter(), left_gravity=True)
        buf.insert(buf.get_end_iter(), retweet_prefix + retweet_text)

        buf.place_cursor(buf.get_iter_at_mark(mark_c))
        buf.delete_mark(mark_c)

        # Save the retweet replacement mark with right gravity.
        buf.create_mark('retweet-start', buf.get_iter_at_mark(mark_rts))
        buf.delete_mark(mark_rts)

    def configure_event(self, w, e):
        """ Store the location of the window, so that if we're closed we
        can re-open where we were; only stored in memory for now, not to
        disk, so on start-up will always have the normal size and automatic
        position."""
        # Note: can't use (e.x, e.y) because of window decorations; the
        # get_position() method apparently contains something terrible to
        # work around that.
        FeedView.win_pos = self.window.get_position()
        FeedView.win_size = (e.width, e.height)

    def destroy(self, w, data=None):
        self.quitter = None
        self.model.unsubscribe_view(self)
        for u in self.updaters:
            u.do_quit()

    def close(self):
        # Called by statusicon.py when the icon is clicked.
        # Nothing special here, just destroy the window...
        self.window.destroy()


time_table = (
    (365.2422*24*60*60, 'a year ago', '%d years ago'),
    (30.43685*24*60*60, 'a month ago', '%d months ago'),
    (7*24*60*60, 'a week ago', '%d weeks ago'),
    (24*60*60, 'a day ago', '%d days ago'),
    (60*60, 'an hour ago', '%d hours ago'),
    (60, 'a minute ago', '%d minutes ago'),
    (10, '10s ago', '%d0s ago'),
)

class TimeUpdater:
    def __init__(self, textview, time):
        self.quit = False
        self.time=time
        self.buf = textview.get_buffer()
        pos = self.buf.get_end_iter()
        self.start = self.buf.create_mark(None, pos, left_gravity=True)
        self.end   = self.buf.create_mark(None, pos, left_gravity=False)

        bg = textview.get_style().base[gtk.STATE_NORMAL]
        fg = textview.get_style().text[gtk.STATE_NORMAL]

        my_color = gtk.gdk.Color(
            red = bg.red_float * 0.5 + fg.red_float * 0.5,
            green = bg.green_float * 0.5 + fg.green_float * 0.5,
            blue = bg.blue_float * 0.5 + fg.blue_float * 0.5)

        self.tags = self.buf.create_tag(scale=pango.SCALE_SMALL,
                            style=pango.STYLE_OBLIQUE, foreground=my_color)

        self.update()

    def update(self):
        if self.quit:
            # do not call again - the window's not there any more anyway
            return False
        text, interval = self.convert(now() - self.time)
        text = bidi_protect(text)
        self.buf.delete(self.buf.get_iter_at_mark(self.start),
                                           self.buf.get_iter_at_mark(self.end))
        self.buf.insert_with_tags(self.buf.get_iter_at_mark(self.start),
                                                           text, self.tags)

        glib.timeout_add_seconds(interval, self.update)

        # do not call again - we've rescheduled ourselves explicitly
        return False

    def convert(self, t):
        for unit, singular, plural in time_table:
            if t>0.9*unit:
                count = round(t/unit)
                next = int(unit*(count+1) - t)
                if count==1:
                    return singular, next
                else:
                    return plural % count, next
        if t>=-0.5:
            return 'just now', max(int(10 - t), 1)
        else:
            return '%ds in the future'%round(-t), 1

    def do_quit(self):
        # parent is being destroyed, schedule a quit now
        self.quit = True

if __name__ == "__main__":
    while 1:
        t = float(input('time in seconds:'))
        print time_to_when(t)
