# (c) 2009 Jiri Baum
import feed_m, feed_v, msgmenu, statusicon, tubes, tubes_v
import dbus, sys
from dbus.mainloop.glib import DBusGMainLoop
from functools import partial

CONNECTR = 'au.com.baum.connectr'

class DBusApp(dbus.service.Object):
    def __init__(self, app):
        self.app = app
        self._interfaces = set([CONNECTR])
        service_name = CONNECTR
        object_path = '/'+service_name.replace('.', '/')

        bus = dbus.SessionBus()
        dbus.service.Object.__init__(self, bus, object_path)
        try:
            self.name_ownership = dbus.service.BusName(service_name,
                                                bus=bus, do_not_queue=True)
        except dbus.exceptions.NameExistsException:
            other = bus.get_object(service_name, object_path)
            other.activate()
            sys.exit()

    @dbus.service.method(dbus_interface=CONNECTR,
                                         in_signature='', out_signature='')
    def activate(self):
        self.app.activate(toggle=False)


class App:
    def __init__(self):
        DBusGMainLoop(set_as_default=True)

        self.dbus_wrapper = DBusApp(self)

        feed_model = feed_m.FeedModel()
        feed_view = feed_v.FeedView(feed_model)
        tubes_model = tubes.Tubes((feed_model, ))

        self.main_win = feed_view
        self.main_win_factory = partial(feed_v.FeedView, feed_model)

        self.si = statusicon.StatusIcon(
            self,
            partial(tubes_v.TubesView, tubes_model))

        self.msgmenu = msgmenu.MsgMenu(self)

    def activate(self, toggle=False, when=None):
        if self.main_win is None or self.main_win.quitter is None:
            self.main_win = self.main_win_factory()
        elif self.main_win.window.has_toplevel_focus():
            if toggle:
                self.main_win.close()
                self.main_win = None
        elif when==None:
            self.main_win.window.present()
        else:
            self.main_win.window.present_with_time(when)
