#!/usr/bin/python
# (c) 2009 Jiri Baum

from __future__ import with_statement

import colorsys, os, tempfile
import pygtk
pygtk.require('2.0')
import glib, gtk, pango

from time import time as now
from utils import *

class TubesView:
    win_size = (400, 500)

    def __init__(self, model):
        self.model = model

        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.window.set_title('connectr - tube status')
        self.window.set_default_size(*self.win_size)
        icon_file = '/usr/share/icons/hicolor/scalable/apps/connectr.svg'
        self.window.set_icon_from_file(icon_file)
        self.window.connect("destroy", self.destroy)
        self.window.connect("delete_event", lambda *x: False)
        self.quitter = quitter()

        self.scroller = gtk.ScrolledWindow()
        self.window.add(self.scroller)

        self.update()

        self.window.show_all()

        self.model.subscribe_view(self)

    def tint(self, w, tint_id):
        bg = w.get_style().base[gtk.STATE_NORMAL]
        fg = w.get_style().text[gtk.STATE_NORMAL]

        new_bg = [int(65535*x+0.5) for x in colorsys.hsv_to_rgb(
            (hash(tint_id)%256) / 256.0,
            max(0.1, bg.saturation * 0.9 + fg.saturation * 0.1),
            bg.value * 0.9 + fg.value * 0.1,
        )]
        new_bg = gtk.gdk.Color(*new_bg)
        w.modify_base(gtk.STATE_NORMAL, new_bg)

    def update(self):
        msgs = sorted(self.model.messages.items())
        if not msgs:
            msgs = [(('none', 'tubes'), 'No tube status')]
        table = gtk.Table(rows=len(msgs), columns=2)
        for i, ((typ, name), msg) in enumerate(msgs):
            name_w = gtk.TextView()
            name_w.get_buffer().set_text(name)
            self.tint(name_w, typ)
            msg_w = gtk.TextView()
            msg_w.get_buffer().set_text(msg)
            self.tint(msg_w, msg)
            table.attach(name_w, 0, 1, i, i+1)
            table.attach(msg_w, 1, 2, i, i+1)
        table.show_all()

        old = self.scroller.get_child()
        if old:
            self.scroller.remove(old)
        self.scroller.add_with_viewport(table)

    def destroy(self, w, data=None):
        self.quitter = None
        self.model.unsubscribe_view(self)

    def close(self):
        # Called by statusicon.py when the icon is clicked.
        # Nothing special here, just destroy the window...
        self.window.destroy()


time_table = (
    (365.2422*24*60*60, 'a year ago', '%d years ago'),
    (30.43685*24*60*60, 'a month ago', '%d months ago'),
    (7*24*60*60, 'a week ago', '%d weeks ago'),
    (24*60*60, 'a day ago', '%d days ago'),
    (60*60, 'an hour ago', '%d hours ago'),
    (60, 'a minute ago', '%d minutes ago'),
    (10, '10s ago', '%d0s ago'),
)

class TimeUpdater:
    def __init__(self, textview, time):
        self.quit = False
        self.time=time
        self.buf = textview.get_buffer()
        pos = self.buf.get_end_iter()
        self.start = self.buf.create_mark(None, pos, left_gravity=True)
        self.end   = self.buf.create_mark(None, pos, left_gravity=False)

        bg = textview.get_style().base[gtk.STATE_NORMAL]
        fg = textview.get_style().text[gtk.STATE_NORMAL]

        my_color = gtk.gdk.Color(
            red = bg.red_float * 0.5 + fg.red_float * 0.5,
            green = bg.green_float * 0.5 + fg.green_float * 0.5,
            blue = bg.blue_float * 0.5 + fg.blue_float * 0.5)

        self.tags = self.buf.create_tag(scale=pango.SCALE_SMALL,
                            style=pango.STYLE_OBLIQUE, foreground=my_color)

        self.update()

    def update(self):
        if self.quit:
            # do not call again - the window's not there any more anyway
            return False
        text, interval = self.convert(now() - self.time)
        self.buf.delete(self.buf.get_iter_at_mark(self.start),
                                           self.buf.get_iter_at_mark(self.end))
        self.buf.insert_with_tags(self.buf.get_iter_at_mark(self.start),
                                                           text, self.tags)

        glib.timeout_add_seconds(interval, self.update)

        # do not call again - we've rescheduled ourselves explicitly
        return False

    def convert(self, t):
        for unit, singular, plural in time_table:
            if t>0.9*unit:
                count = round(t/unit)
                if count==1:
                    return singular, unit
                else:
                    return plural % count, unit
        if t>=-0.5:
            return 'just now', 10 - t
        else:
            return '%ds in the future'%round(-t), 1

    def do_quit(self):
        # parent is being destroyed, schedule a quit now
        self.quit = True

if __name__ == "__main__":
    while 1:
        t = float(input('time in seconds:'))
        print time_to_when(t)
