# (c) 2009 Jiri Baum

from functools import partial
import unicodedata

class cb_call(object):
    """ The cb_call object is useful when a callback should ignore the
    parameters, call a single function, ignore the result, and return True.
    Based on the partial() code from PEP 309. 

        cb_call(fun, *args1, **kwargs1)(*args2, **kwargs2)
    is equivalent to:
        fun(*args1, **kwargs1)
    """
    def __init__(*args, **kw):
        self = args[0]
        self.fn, self.args, self.kw = (args[1], args[2:], kw)

    def __call__(self, *args_discard, **kw_discard):
        self.fn(*self.args, **self.kw)
        return True

class quitter(object):
    """ Each class which has a top-level window (or a system tray icon)
    should hold an instance of this object as long as the window is up.
    When the last instance goes away, the quitter calls gtk.main_quit().
    """
    count = 0
    def __init__(self):
        self.__class__.count += 1
    def __del__(self):
        import gtk
        self.__class__.count -= 1
        if not self.__class__.count:
            gtk.main_quit()

def cfgfilename(base):
    import os
    cfgdir = os.path.expanduser('~/.connectr')
    if not os.path.exists(cfgdir):
        os.mkdir(cfgdir)
    return os.path.join(cfgdir, base)

def bidi_para_dir(para):
    """ Implements the "Paragraph Level" part of the Unicode Bidirectional
    Algorithm on a single paragraph (steps P2 and P3), returning 'R' if the
    level is one and 'L' if it is zero. """
    for ch in unicode(para):
        dir = unicodedata.bidirectional(ch)
        if dir in ('R', 'AL'):
            return 'R'
        if dir in ('L'):
            return 'L'
    return 'L' # default specified in Unicode, step P3 of the algorithm

def bidi_protect(span):
    """ Helps avoid a given span of text getting intermixed with adjacent
    but logically separate spans of text by ensuring that the first and
    last characters have strong BIDI directions rather than being neutrals
    or numbers or something (and that they're the same direction).

    It's not perfect, but I'm not sure what's the Right Thing to do. One
    additional step it could do is to also mark the whole text as an
    embedding (of the same direction), but I don't think that's the Right
    Thing either. """

    #prot = {'L': u'\u200E%s\u200E',
    #        'R': u'\u200F%s\u200E'}[bidi_para_dir(span)]
    prot = {'L': u'\u202A%s\u202C',
            'R': u'\u202B%s\u202C'}[bidi_para_dir(span)]
    return prot%span
